package MT::Plugin::TemplateSelector;
use strict;
use MT;
use MT::Template;

require MT::Plugin;

our $VERSION = 1.0;
use base qw( MT::Plugin );

my $plugin = MT::Plugin::TemplateSelector->new({
    id => 'TemplateSelector',
    key => 'template-selector',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    name => 'TemplateSelector',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    'version' => $VERSION,
    l10n_class => 'TemplateSelector::L10N',
    schema_version => 0.92,
});

MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'entry' => {
                'sel_template_name' => 'string(255)',
            },
            'template' => {
                'is_selector' => 'integer',
            },
        },
        callbacks => {
            'MT::App::CMS::template_source.edit_entry'
                => \&_add_template_selector,
            'MT::App::CMS::template_source.edit_template'
                => \&_selector_check,
            'MT::App::CMS::template_param.edit_template'
                => \&_selector_flag,
        },
        tags => {
            block => {
                'TemplateSelector?' => \&_template_selector,
                'IfTemplateSelector?' => \&_template_selector,
            },
        }
   });
}

sub _add_template_selector {
    my ($cb, $app, $tmpl) = @_;
    my $blog_id = $app->blog->id;
    my $q = $app->param;
    my $id = $q->param('id');
    my $template_options = '<mt:include name="include/actions_bar.tmpl" bar_position="bottom" hide_pager="1">';
    my $label = $plugin->translate("Template");
    my $iter = MT::Template->load_iter({ blog_id => $blog_id,
                                         type => 'custom' });
    my $entry; my $entry_tmplate;
    if ($id) {
        $entry = MT::Entry->load({ id => $id });
        $entry_tmplate = $entry->sel_template_name;
    }
    my $selector; my $counter = 0;
    while (my $tmp = $iter->()) {
        my $tname = $tmp->name;
        my $tid = $tmp->id;
        if ($tmp->is_selector) {
            $counter++;
            if ($entry_tmplate eq $tname) {
                $selector .= '<option value="'.$tname.'" selected="selected">'.$tname.'</option>'."\n";
            } else {
                $selector .= '<option value="'.$tname.'">'.$tname.'</option>'."\n";
            }
        }
    }
    if ($counter) {
        my $new = <<"HTML";
<fieldset class="pkg">
<mtapp:setting
    id="template_selector"
    label="$label"
    >
    <ul>
        <li><select name="sel_template_name" id="sel_template_name" class="full-width" onchange="highlightSwitch(this)">
            $selector
            </select></li>
    </ul>
    <input type="hidden" name="type" value="custom" />
</mtapp:setting>
</fieldset>
HTML
        my $search = quotemeta($template_options);
        $$tmpl =~ s/$search/$new$template_options/s;
    }
}

sub _selector_check {
    my ($cb, $app, $tmpl) = @_;
    my $q = $app->param;
    my $id = $q->param('id');
    my $type = $q->param('type');
    my $template;
    if ($id) {
        $template = MT::Template->load({id => $id});
        $type = $template->type;
    }
    if (($type eq 'custom') || ($type eq 'module')) {
        my $template_options = '<mt:include name="include/actions_bar.tmpl" bar_position="bottom" hide_pager="1">';
        my $label = $plugin->translate("Template Selector");
        my $caption = $plugin->translate("Add this template to Template Selector.");
        my $new = <<"HTML";
    <mtapp:setting
        id="is_selector"
        label="$label"
        >
        <ul>
            <li><input type="checkbox" id="is_selector" name="is_selector" value="1" <mt:if name="is_selector">checked="checked" </mt:if> mt:watch-change="1" /> <label for="is_selector">$caption</label></li>
        </ul>
        <input type="hidden" name="is_selector" value="0" />
    </mtapp:setting>
HTML
        my $search = quotemeta($template_options);
        $$tmpl =~ s/$search/$new$template_options/s;
    }
}

sub _selector_flag {
    my ($cb, $app, $param, $tmpl) = @_;
    my $q = $app->param;
    my $id = $q->param('id');
    if ($id) {
        my $template = MT::Template->load({id => $id});
        $param->{is_selector} = $template->is_selector;
    }
}

sub _template_selector {
    my ($ctx, $args, $cond) = @_;
    my $entry = $ctx->stash('entry') or return $ctx->_no_entry_error('TemplateSelector');
    my $name = $args->{name};
    my $entry_tmplate = $entry->sel_template_name;
    if ($name eq $entry_tmplate) {
        return 1;
    }
    return 0;
}

1;